( function( blocks, editor, element, components ) {
    const __ = wp.i18n.__; // The __() for internationalization.
    const el = element.createElement; // The wp.element.createElement() function to create elements.
    const registerBlockType = blocks.registerBlockType; // The registerBlockType() to register blocks.
    const { SelectControl } = components;
    const withSelect = wp.data.withSelect;

    function Icon() {
	  return el(
	    'svg',
	    {
	      xmlns: 'http://www.w3.org/2000/svg',
	      width: '41',
	      height: '41',
	      viewBox: '0 0 41 41'
	    },
	    el(
	      'g',
	      { fill: 'none', fillRule: 'evenodd', stroke: 'none', strokeWidth: '1' },
	      el('path', { d: 'M0 0H41V41H0z' }),
	      el(
	        'g',
	        { stroke: '#60666A', transform: 'translate(7 9)' },
	        el('path', { d: 'M2.5 0h22a2.5 2.5 0 110 5h-22a2.5 2.5 0 010-5zM2.5 9h22a2.5 2.5 0 110 5h-22a2.5 2.5 0 110-5z' }),
	        el('path', { fill: '#60666A', d: 'M2 18h10a2 2 0 110 4H2a2 2 0 110-4z' })
	      )
	    )
	  );
	}

    registerBlockType( 'piotnetforms/form', { // Block name; must be a string that contains a namespace prefix followed by a forward slash.
        title: __( 'Form', 'piotnetforms' ),
        icon: el(Icon, null),
        category: 'piotnetforms', // Group blocks together based on common traits, can be 'common', 'formatting', 'layout', 'widgets' or 'embed'.
        keywords: [ 'form', 'piotnet', 'piotnetforms' ], // Keywords for custom block, limited to three.
        attributes: { // Placeholders to store custom block setting values.
            form_id: {
                type: 'string',
            },
        },
        edit: withSelect( function( select, props ) {
	            var form_ids = select('core').getEntityRecords('postType', 'piotnetforms', { per_page: -1 });

	            return {
	                form_ids: form_ids
	            };
	        } )
        	( function( props ) {

        		var options = [];
		
				if( props.form_ids ) {
					options.push( { value: '', label: 'Select a form' } );
					props.form_ids.forEach((post) => {
						options.push({value:post.id, label:post.title.rendered});
					});
				} else {
					options.push( { value: '', label: 'Loading...' } )
				}

	            return [
	                el( 'div',
	                    {
	                        key: 'piotnetforms-gutenberg', // React requires each top-level item in the returned array to have a unique key.
	                        className: 'piotnetforms-gutenberg', // className property is automatically enabled for each block.
	                    },
	                    el( SelectControl,
					        {
					            label: __( 'Form - Piotnetforms', 'piotnetforms' ),
					            options: options,
					            onChange: ( value ) => {
					                props.setAttributes(
					                    {
					                        form_id: value
					                    }
					                );
					            },
					            value: props.attributes.form_id
					        }
					    )
	                )
	            ];
	        } )
    });
}(
    window.wp.blocks,
    window.wp.blockEditor,
    window.wp.element,
    window.wp.components
) );