import '../lib/jquery-nestable.js';

export default class Navigator {
    serialized_before = null;

    constructor(pb, editorSetting, $, $iframe, changeCallback, activeCallback, clickEditCallback) {
        this.pb = pb;
        this.editorSetting = editorSetting;

        this.$ = $;
        this.$navigator = this.$(document).find('.piotnetforms-navigator-widgets');

        this.$iframe = $iframe;
        this.$preview = $iframe.find('[data-piotnetforms-widget-preview]');
        this.changeCallback = changeCallback;
        this.activeCallback = activeCallback;

        $(document).find('.piotnetforms-navigator').draggable({ handle: '.piotnetforms-navigator-header' });
        $(document).on('click', '[data-piotnetforms-navigator-toggle]', this.on_click_toggle.bind(this));
        $(document).on('click', '.piotnetforms-navigator-close', this.on_click_toggle.bind(this));
        $(document).on('click', '[data-navigator-piotnetforms-widget-id]', clickEditCallback);

        if (this.editorSetting.isNavigatorShow()) {
            this.on_click_toggle();
        }
    }

    trim_content(content) {
        content = content.trim().replace(/\s\s+/g, ' ');
        if (content.length > 20) {
            content = content.substring(0, 20).trim() + "...";
        }
        return content;
    }

    getFieldTypeBeauty(fieldType) {
        const controls = this.pb.get_widget_info('field')['structure'][0]['sections'][0]['controls'];

        for (const control of controls) {
            if (control['name'] === "field_type") {
                const options = control['options'];
                if (options[fieldType]) {
                    return options[fieldType];
                }
            }
        }

        return fieldType;
    }

    get_title_widget(widgetId) {
        const $widget = this.$preview.find('[data-piotnet-editor-widgets-item-id="' + widgetId + '"]');

        const widgetInfo = JSON.parse($widget.attr('data-piotnet-editor-widgets-item'));
        let widgetType = widgetInfo['title'];

        const $navigatorWidget = $widget.find('.' + widgetId);
        let content = this.trim_content($navigatorWidget.text());

        if (widgetType === 'Space' || widgetType === 'Multi Step Start') {
            content = null;
        } else if (widgetType === 'Field') {
            const fieldType = this.pb.get_setting_widget(widgetId)['fields']['field_type'];
            const fieldTypeBeauty =  this.getFieldTypeBeauty(fieldType)
            widgetType = 'Field (' + fieldTypeBeauty + ")";
        } else if (widgetType === 'Multi Step End') { // Remove msg: The form was sent successfully.
            const $clone = $navigatorWidget.clone();
            $clone.find('.piotnetforms-alert').remove();
            content = (this.trim_content($clone.text()))
        } else if (widgetType === 'Submit') { // Remove msg: The form was sent successfully.
            const $clone = $navigatorWidget.find('.piotnetforms-button-text');
            content = (this.trim_content($clone.text()))
        }

        if (content) {
            return  '<b>' + widgetType + ': </b>' + content;
        } else {
            return  '<b>' + widgetType + '</b>';
        }
    }

    update_title_widget(widgetId) {
        if (widgetId === 'single-settings' || widgetId === 'form-global') {
            return;
        }
        const $title = this.$('[data-navigator-piotnetforms-widget-id="' + widgetId + '"]').find('.piotnetforms-navigator-widget-handle-title').first();
        const title = this.get_title_widget(widgetId);
        $title.html(title);
    }

    build_navigator_html(widget) {
        const widgetId = widget['id'];
        const $ = this.$;

        const widgetType = widget['class_name'].replace("piotnetforms_", "").replace("Piotnetforms_", "");
        const widget_info = this.pb.get_widget_info_by_class_name(widget['class_name']);
        const widgetTitle = this.get_title_widget(widgetId);

        const noChildren = widgetType === 'Section' || widgetType === 'Column' ? '' : 'piotnetforms-navigator-widget-nochildren';

        const subElements = widget['elements'];
        const hasSubElements = subElements && subElements.length > 0;
        const innerHTMLs = [];
        if (hasSubElements) {
            for (const subElement of subElements) {
                innerHTMLs.push(this.build_navigator_html(subElement));
            }
        }
        const innerHTMLJoined = noChildren === '' ? '<ol class="piotnetforms-navigator-widget-list">' + innerHTMLs.join("\n") + '</ol>': '';

        const pluginURL = $('[data-piotnetforms-plugin-url]').attr('data-piotnetforms-plugin-url') + '/piotnetforms-pro';

        return `<li class="piotnetforms-navigator-widget ${noChildren}" data-navigator-piotnetforms-widget-id="${widgetId}" data-navigator-piotnetforms-widget-type="${widgetType}">
            <div class="piotnetforms-navigator-widget-handle">
                <div class="piotnetforms-navigator-widget-handle-title">${widgetTitle}</div>
                <div class="piotnetforms-navigator-widget-handle-controls">
                    <div class="piotnetforms-navigator-widget-handle-controls-item piotnetforms-navigator-widget-handle-controls-item--duplicate" title="Duplicate" data-navigator-piotnetforms-widget-control="duplicate">
                        <img src="${pluginURL}/assets/icons/e-control-duplicate.svg"/>
                    </div>
                    <div class="piotnetforms-navigator-widget-handle-controls-item piotnetforms-navigator-widget-handle-controls-item--remove" title="Delete" data-navigator-piotnetforms-widget-control="remove">
                        <img src="${pluginURL}/assets/icons/e-control-remove.svg"/>
                    </div>
                </div>
            </div>
            ${innerHTMLJoined}
        </li>`;
    }

    process_when_drop_widget(mainWidgetId, widgetId, tree_widgets) {
        for (let i = 0; i < tree_widgets.length; i++) {
            const widget = tree_widgets[i];
            if (widget['id'] === widgetId) {
                if (tree_widgets.length === 1) {
                    // Tìm thấy bên trong chỉ có duy nhất 1 widget
                    if (mainWidgetId) {
                        const $mainNavigator = this.$('[data-navigator-piotnetforms-widget-id="' + mainWidgetId + '"]');
                        const navigator_html = this.build_navigator_html(widget);
                        const widgetList = $mainNavigator.find('.piotnetforms-navigator-widget-list');
                        if (widgetList.length === 0) {
                            $mainNavigator.append('<ol class="piotnetforms-navigator-widget-list">' + navigator_html + '</ol>')
                        } else {
                            $mainNavigator.find('.piotnetforms-navigator-widget-list').html(navigator_html);
                        }
                    } else {
                        const $mainNavigator = this.$('.piotnetforms-navigator-widget-root-list');
                        const navigator_html = this.build_navigator_html(widget);
                        $mainNavigator.html(navigator_html);
                    }
                } else if (i === 0) {
                    // Nằm đầu tiên
                    const beforeWidgetId = tree_widgets[i + 1]['id']
                    const $beforeWidget = this.$('[data-navigator-piotnetforms-widget-id="' + beforeWidgetId + '"]');
                    const navigator_html = this.build_navigator_html(widget);
                    $beforeWidget.before(navigator_html);
                } else {
                    const afterWidgetId = tree_widgets[i - 1]['id'];
                    const $afterWidget = this.$('[data-navigator-piotnetforms-widget-id="' + afterWidgetId + '"]');
                    const navigator_html = this.build_navigator_html(widget);
                    $afterWidget.after(navigator_html);
                }
                return true;
            }

            const subWidgets = widget['elements'];
            if (subWidgets && subWidgets.length > 0) {
                if (this.process_when_drop_widget(widget['id'], widgetId, subWidgets)) {
                    return true;
                }
            }
        }
        return false;
    }

    rebuild_when_drop_widget(widgetId) {
        const tree_widgets = this.pb.get_tree_setting_widgets();
        this.$('[data-navigator-piotnetforms-widget-id="' + widgetId + '"]').remove();
        this.process_when_drop_widget(null, widgetId, tree_widgets);
        this.active_widget(widgetId);
        this.$navigator.nestable('initParents');
    }

    rebuild_when_duplicate_widget(widget_id, new_widget_id) {
        var tree_widget = pb.get_tree_setting_widget(new_widget_id);
        var widget = this.$('[data-navigator-piotnetforms-widget-id="' + widget_id + '"]');
        widget.after(this.build_navigator_html(tree_widget));
        this.$navigator.nestable('initParents');
    }

    on_click_toggle() {
        const $piotnetformsNavigator = this.$(document).find('.piotnetforms-navigator');
        $piotnetformsNavigator.toggleClass('piotnetforms-navigator-hidden');
        const status = $piotnetformsNavigator.hasClass('piotnetforms-navigator-hidden');
        this.editorSetting.setNavigatorShow(!status);
    }

    find_position_in_nestable(data, widget_id, parent_id) {
        const length = data.length;
        for (let i = 0; i < length; i++) {
            const widget = data[i];
            const current_id = widget['navigatorPiotnetformsWidgetId'];

            if (current_id === widget_id) {
                const before_id = i > 0 ? data[i - 1]['navigatorPiotnetformsWidgetId'] : null;
                const after_id = i < length - 1 ? data[i + 1]['navigatorPiotnetformsWidgetId'] : null;
                return {'index': i, 'parent_id': parent_id, 'before_id': before_id, 'after_id': after_id};
            }
            if (widget['children']) {
                const sub = this.find_position_in_nestable(widget['children'], widget_id, current_id);
                if (sub != null) {
                    return sub;
                }
            }
        }

        return null;
    }

    build_new_navigator(skipInit) {
        const self = this;

        const widgets = this.pb.get_tree_setting_widgets();

        const htmls = [];
        for (const widget of widgets) {
            htmls.push(this.build_navigator_html(widget));
        }

        self.$navigator.html("<ol class=\"piotnetforms-navigator-widget-list piotnetforms-navigator-widget-root-list\">" + htmls.join("\n") + "</ol>");

        if(skipInit) {
            return;
        }

        const $ = this.$;
        self.$navigator.nestable({
            'maxDepth': 10000,
            'rootClass': 'piotnetforms-navigator-widgets',
            'listClass': 'piotnetforms-navigator-widget-list',
            'itemClass': 'piotnetforms-navigator-widget',
            'dragClass': 'piotnetforms-navigator-widget-dragel',
            'handleClass': 'piotnetforms-navigator-widget-handle',
            'collapsedClass': 'piotnetforms-navigator-widget-collapsed',
            'placeClass': 'piotnetforms-navigator-widget-placeholder',
            'noDragClass': 'piotnetforms-navigator-widget-nodrag',
            'noChildrenClass': 'piotnetforms-navigator-widget-nochildren',
            'emptyClass': 'piotnetforms-navigator-widget-empty',
            'scroll': true,
            onDragStart: function (l, e) {
                self.serialized_before = JSON.stringify(self.$navigator.nestable('serialize'));
            },
            beforeDragStop: function(l,e, p){
                const widgetType = $(e).closest('.piotnetforms-navigator-widget').attr('data-navigator-piotnetforms-widget-type');
                const parentType = p.parent().attr('data-navigator-piotnetforms-widget-type');

                if (widgetType === 'Column') {
                    return parentType === 'Section';
                } else if (parentType === 'Column') {
                    return true;
                } else if (parentType === undefined) {
                    return true;
                } else {
                    return false;
                }
            },
            callback: function(l,e){
                const widget_id = $(e).attr('data-navigator-piotnetforms-widget-id');

                const data = self.$navigator.nestable('serialize')
                const serialized_current = JSON.stringify(data);
                if (serialized_current === self.serialized_before)
                {
                    self.activeCallback(widget_id);
                    return;
                }

                const widget = self.$iframe.find('[data-piotnet-editor-widgets-item-id="' + widget_id + '"]');
                const position_data = self.find_position_in_nestable(data, widget_id);

                const before_id = position_data['before_id'];
                const after_id = position_data['after_id'];
                const parent_id = position_data['parent_id'];
                if (before_id) {
                    const previewBeforeWidget = self.$iframe.find('[data-piotnet-editor-widgets-item-id="' + before_id + '"]');
                    previewBeforeWidget.after(widget);
                } else if (after_id) {
                    const previewAfterWidget = self.$iframe.find('[data-piotnet-editor-widgets-item-id="' + after_id + '"]');
                    previewAfterWidget.before(widget);
                } else if (parent_id) {
                    const previewParentWidget = self.$iframe.find('[data-piotnet-editor-widgets-item-id="' + parent_id + '"]');
                    const inner = $(previewParentWidget[0]).find('[data-piotnet-inner-html]');
                    inner.append(widget);
                } else {
                    self.$iframe.find('.piotnetforms-widget-preview').append(widget)
                }

                self.changeCallback(widget_id);
            }
        });

        $('.piotnetforms-navigator-widgets').nestable('collapseAll');

        let navigatorExpanded = false;

        $('.piotnetforms-navigator-expand-toggle').on('click', function () {
            $(this).toggleClass('active');

            if (!navigatorExpanded) {
                $('.piotnetforms-navigator-widgets').nestable('expandAll');
                navigatorExpanded = true;
            } else {
                $('.piotnetforms-navigator-widgets').nestable('collapseAll');
                navigatorExpanded = false;
            }
        });

        $(document).on('click','[data-navigator-piotnetforms-widget-control]',function(){
            let action = $(this).attr('data-navigator-piotnetforms-widget-control'),
                widget_id = $(this).closest('[data-navigator-piotnetforms-widget-id]').attr('data-navigator-piotnetforms-widget-id'),
                $widget = self.$iframe.find('[data-piotnet-editor-widgets-item-id="' + widget_id + '"]');

            switch (action) {
                case 'duplicate':
                    $widget.find('[data-piotnet-control-duplicate]').first().trigger('click');
                    break;

                case 'remove':
                    $widget.find('[data-piotnet-control-remove]').first().trigger('click');
                    break;
            }
        });
    }

    active_widget(widget_id) {
        const $ = this.$;

        this.$navigator.find('[data-navigator-piotnetforms-widget-id]').removeClass('active');
        const widget = this.$navigator.find('[data-navigator-piotnetforms-widget-id="' + widget_id + '"]');
        widget.addClass('active');

        // this.$navigator.nestable('expandItem', widget);
        widget.parents('.piotnetforms-navigator-widget').each(function () {
            $(this).removeClass('piotnetforms-navigator-widget-collapsed')
        });
    }

    deactivate_widget() {
        this.$navigator.find('[data-navigator-piotnetforms-widget-id]').removeClass('active');
    }

    remove_widget(widget_id) {
        this.$navigator.find('[data-navigator-piotnetforms-widget-id="' + widget_id + '"]').remove();
    }

    get_html() {
        return this.$navigator.html();
    }

    set_html(html) {
        this.$navigator.html(html);
    }
}