export default class UndoRedoHandler {
    currentIndex = 0;
    queue = [];

    disabled_button_class_name = "piotnetforms-builder__header-button--disabled";
    locked_button_class_name = "piotnetforms-builder__header-button--locked";
    debounce = null;

    current_action_type = null;
    current_widget_id = null;

    constructor(pb, navigator, $, $iframe) {
        this.pb = pb;
        this.navigator = navigator;
        this.$ = $;

        this.$iframe = $iframe;
        this.$preview = $iframe.find('[data-piotnetforms-widget-preview]');
        this.$head = $iframe.find('head').first();
        this.$panelWidgetSettings = $("[data-piotnetforms-editor-widget-settings]");

        this.undo_disabled = true;
        this.undo_ele = $('[data-piotnetforms-builder-button="undo"]');
        this.undo_ele.on('click', this.undo.bind(this));

        this.redo_disabled = true;
        this.redo_ele = $('[data-piotnetforms-builder-button="redo"]');
        this.redo_ele.on('click', this.redo.bind(this));

        // Add Shortcut: Ctrl + Z and Ctrl + Y
        $(document).keydown(this.handle_key_down.bind(this));
        $iframe.keydown(this.handle_key_down.bind(this));

        const action_data = this.build_action_data();
        this.queue.push(action_data);
        this.currentIndex++;
    }

    set_current_action(action_type, widget_id) {
        this.current_action_type = action_type;
        this.current_widget_id = widget_id;
    }

    add_action2() {
        this.add_action(this.current_action_type, this.current_widget_id);
    }

    latest_add_action = new Date().getTime();

    add_action(action_type, widget_id) {
        const now = new Date().getTime();
        // console.log('hit add action', now - this.latest_add_action, action_type, widget_id);
        // this.latest_add_action = now;

        const action = this.build_action_data();
        action['data_time'] = now;
        action['widget_id'] = widget_id;
        action['action_type'] = action_type;

        action['action_name'] = this.build_action_name(action);

        if (action_type === 'CHANGE_POSITION_WIDGET') {
            if (action.tree_setting_widgets === this.queue[this.currentIndex - 1].tree_setting_widgets) { // Don't change position
                return;
            }
        } else if (action_type === 'PASTE_STYLE') {
            if (JSON.stringify(action.setting_widgets_obj[widget_id]) === JSON.stringify(this.queue[this.currentIndex - 1].setting_widgets_obj[widget_id])) { // Don't change style
                return;
            }
        } else if (action_type === 'CREATE_WIDGET') {
            if (action_type === this.queue[this.currentIndex - 1].action_type && widget_id === this.queue[this.currentIndex - 1].widget_id) {
                this.queue[this.currentIndex - 1] = action;
                // console.log('overwrite');
                return;
            }
        } else if (action_type === 'CHANGE_SETTING_WIDGET') {
            if (JSON.stringify(action.setting_widgets_obj[widget_id]) === JSON.stringify(this.queue[this.currentIndex - 1].setting_widgets_obj[widget_id])) {
                this.queue[this.currentIndex - 1] = action;
                // console.log('overwrite');
                return;
            }
        }

        clearTimeout(this.debounce);
        this.debounce = setTimeout(function () {
            if (this.currentIndex < this.queue.length) {
                this.queue = this.queue.slice(0, this.currentIndex);
            }

            this.queue.push(action);
            this.currentIndex++;
            this.update_state_undo_redo_button();

            // console.log('add action', 'queue' , this.queue.length, 'currentIndex', this.currentIndex, action);
        }.bind(this), 500);
    }

    build_action_name(action) {
        const widget_id = action['widget_id'];
        if (action.setting_widgets_obj[widget_id] !== undefined) {
            const widget_name = this.pb.get_widget_info(action.setting_widgets_obj[widget_id].type)['title'];
            switch (action['action_type']) {
                case "CREATE_WIDGET": return `Create new ${widget_name} widget with ID ${widget_id}`;
                case "CHANGE_SETTING_WIDGET": return `Change setting of ${widget_name} widget with ID ${widget_id}`;
                case "DUPLICATE_WIDGET": return `Duplicate ${widget_name} widget from ID ${widget_id}`;
                case "REMOVE_WIDGET": return `Remove ${widget_name} widget with ID ${widget_id}`;
                case "CHANGE_POSITION_WIDGET": return `Change position of ${widget_name} widget with ID ${widget_id}`;
                case "PASTE_STYLE": return `Paste style of ${widget_name} widget with ID ${widget_id}`;
                default: return "Unknown action";
            }
        }
    }

    undo() {
        if (this.currentIndex <= 1) {
            return;
        }

        this.lock_button();

        this.currentIndex--;
        this.restore_state(true, this.queue[this.currentIndex - 1], this.queue[this.currentIndex]);

        this.update_state_undo_redo_button();
        // console.log('undo', 'queue' , this.queue.length, 'currentIndex', this.currentIndex);

        this.unlock_button();
    }

    redo() {
        if (this.currentIndex >= this.queue.length) {
            return;
        }

        this.lock_button();

        this.currentIndex++;
        this.restore_state(false, this.queue[this.currentIndex - 2], this.queue[this.currentIndex - 1]);

        this.update_state_undo_redo_button();
        // console.log('redo', 'queue' , this.queue.length, 'currentIndex', this.currentIndex);

        this.unlock_button();
    }

    update_state_undo_redo_button() {
        // Bật tắt button Undo
        if (this.currentIndex > 1) {
            if (this.undo_disabled) {
                this.undo_ele.removeClass(this.disabled_button_class_name);
                this.undo_disabled = !this.undo_disabled;
            }
        } else {
            if (!this.undo_disabled) {
                this.undo_ele.addClass(this.disabled_button_class_name);
                this.undo_disabled = !this.undo_disabled;
            }
        }

        // Bật tắt button Redo
        if (this.currentIndex < this.queue.length) {
            if (this.redo_disabled) {
                this.redo_ele.removeClass(this.disabled_button_class_name);
                this.redo_disabled = !this.redo_disabled;
            }
        } else {
            if (!this.redo_disabled) {
                this.redo_ele.addClass(this.disabled_button_class_name);
                this.redo_disabled = !this.redo_disabled;
            }
        }
    }

    handle_key_down(e) {
        if (e.ctrlKey) {
            if (e.keyCode === 90) { // undo
                this.undo();
            }
            if (e.keyCode === 89) { // redo
                this.redo();
            }
        }
    }

    lock_button() {
        this.undo_ele.addClass(this.locked_button_class_name);
        this.redo_ele.addClass(this.locked_button_class_name);
    }

    unlock_button() {
        this.undo_ele.removeClass(this.locked_button_class_name);
        this.redo_ele.removeClass(this.locked_button_class_name);
    }

    build_action_data() {
        // console.time('setting_widgets');
        const setting_widgets = JSON.stringify(this.pb.get_setting_widgets());
        // console.timeEnd('setting_widgets');
        // console.time('tree_setting_widgets');
        const tree_setting_widgets = JSON.stringify(this.pb.get_tree_setting_widgets());
        // console.timeEnd('tree_setting_widgets');
        // console.time('css');
        const css = JSON.stringify(this.pb.get_css_widgets());
        // console.timeEnd('css');
        // console.time('global');
        const global = JSON.stringify(this.pb.master.global);
        // console.timeEnd('global');
        // console.time('single');
        const single = JSON.stringify(this.pb.master.single);
        // console.timeEnd('single');
        // console.time('html');
        const html = this.$preview.clone().html();
        // console.timeEnd('html');
        // console.time('navigator');
        const navigator = this.navigator.get_html();
        // console.timeEnd('navigator');

        // console.time('setting_widgets_obj');
        const setting_widgets_obj = JSON.parse(setting_widgets);
        // console.timeEnd('setting_widgets_obj');

        return {
            setting_widgets: setting_widgets,
            tree_setting_widgets: tree_setting_widgets,
            css: css,
            global: global,
            single: single,
            html: html,
            navigator: navigator,
            setting_widgets_obj: setting_widgets_obj,
        }
    }

    // TODO bên Editor cũng có
    remove_global_settings_edited() {
        this.$(document).find('[data-piotnetforms-global-settings-edited]').removeAttr('data-piotnetforms-global-settings-edited');
        this.$iframe.find('[data-piotnetforms-global-settings-edited]').removeAttr('data-piotnetforms-global-settings-edited');

        this.$(document).find('[data-piotnetforms-single-settings-edited]').removeAttr('data-piotnetforms-single-settings-edited');
        this.$iframe.find('[data-piotnetforms-single-settings-edited]').removeAttr('data-piotnetforms-single-settings-edited');
    }

    // TODO bên Editor cũng có
    back_to_menu() {
        this.$('[data-piotnetforms-editor-header-text]').html('Widgets').removeClass('active');
        this.$('[data-piotnetforms-editor-header-search]').addClass('active');
        this.$('.piotnetforms-settings').scrollTop(0);
        this.$('[data-piotnetforms-editor-widgets-open]').removeClass('active');
        this.$('[data-piotnetforms-widget-controls]').removeClass('active');
        this.$('[data-piotnetforms-widgets]').addClass('active');
        this.remove_global_settings_edited();
    }

    restore_state(is_undo, beforeState, afterState) {
        const currentState = is_undo ? beforeState : afterState;

        this.pb.set_setting_widgets(JSON.parse(currentState.setting_widgets));
        this.pb.set_tree_setting_widgets(JSON.parse(currentState.tree_setting_widgets));

        const current_css = JSON.parse(currentState.css);
        this.pb.set_css_widgets(current_css);

        this.pb.master.global = JSON.parse(currentState.global);
        this.pb.master.single = JSON.parse(currentState.single);

        // Back to Menu
        this.back_to_menu();

        // Restore preview HTML
        if (beforeState.html !== afterState.html) {
            this.$preview.html(currentState['html']);
            if ( this.$preview.length > 0 && this.$preview.find('[data-piotnet-editor-widgets-item]').length === 0 ) {
                this.$preview.addClass('placeholder');
            } else {
                this.$preview.removeClass('placeholder');
            }

            this.$preview.find('[data-piotnet-editor-widgets-item]').removeClass('active');
        }

        // Restore CSS
        if (beforeState.css !== afterState.css) {
            this.$head.find('[data-piotnet-widget-css-head]').remove();

            const css_style_htmls = [];
            for (const css_id in current_css) {
                css_style_htmls.push('<style data-piotnet-widget-css-head="' + css_id + '">' + current_css[css_id] + '</style>')
            }

            if (css_style_htmls.length > 0) {
                this.$head.append(css_style_htmls.join(""));
            }
        }

        const is_need_remove_control_html = afterState['action_type'] === "CHANGE_SETTING_WIDGET" || afterState['action_type'] === "PASTE_STYLE";
        if (is_need_remove_control_html) {
            const widget_id = afterState['widget_id'];
            // Remove control HTML
            const $widgetSettings = this.$panelWidgetSettings.find('[data-piotnetforms-widget-controls="' + widget_id + '"]');
            $widgetSettings.remove();
            this.pb.set_control_widget(widget_id, null);
        }

        this.navigator.set_html(currentState.navigator);
        this.navigator.deactivate_widget();
    }
}
